<?php
/**
 * @package Joomla
 * @subpackage Abook
 * @copyright (C) 2010-now Federica Ugolotti
 * @license GNU/GPL, see LICENSE.php
 * Abook is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 3
 * as published by the Free Software Foundation.

 * Abook is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Abook; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
// No direct access
defined('_JEXEC') or die;

jimport('joomla.application.component.modellist');
use Joomla\CMS\Factory;
use Joomla\Registry\Registry;
use Joomla\CMS\Language\Multilanguage;
use Joomla\CMS\Language\Text;

class AbookModelTag extends JModelList
{
	protected $_item = null;
	protected $_tag = null;
	//questa funzione contiene i libri che risultano dalla query più in basso, si può anche non mettere, la lascio perchè se voglio fare una variazione alla funzione di default basta che scrivo cose dopo $items = &parent::getItems(); facendo un ciclo for per tutti gli items
	public function getItems()
	{
		// Invoke the parent getItems method to get the main list
		$items = parent::getItems();
		$globalParams = JComponentHelper::getParams('com_abook', true);
		for ($i = 0, $n = count($items); $i < $n; $i++) {
                        $item = &$items[$i];
			//book params
                        $bookParams = new Registry($item->params);
			//item params
                        $item->params = clone $this->getState('params');
			// Create an array of just the params set to 'use_book'
                        $menuParamsArray = $this->getState('params')->toArray();
                        $bookArray    = array();
			foreach ($menuParamsArray as $key => $value)
                                {
                                        if ($value === 'use_book')
                                        {
                                                // If the book has a value, use it
                                                if ($bookParams->get($key) != '')
                                                {
                                                        // Get the value from the book
                                                        $bookArray[$key] = $bookParams->get($key);
                                                }
                                                else
                                                {
                                                        // Otherwise, use the global value
                                                        $bookArray[$key] = $globalParams->get($key);
                                                }
                                        }
                        }
			// Merge the selected article params
                        if (count($bookArray) > 0){
                                $bookParams = new Registry($bookArray);
                                $item->params->merge($bookParams);
                        }
			
                        $this->_authorname($item->id, $item->params);
                        $item->authors=$this->_authorname;
			$this->_tagname($item->id);
                        $item->tags=$this->_tagname;
			$this->_voting($item->id);
                        $item->vote=$this->_voting;
                }
		return $items;
	}

	//query che tira fuori i libri di una categoria che vengono messi nel risultato della funzione getItems
	protected function getListQuery()
	{
		$user   = Factory::getUser();
                $groups = implode(',', $user->getAuthorisedViewLevels());

                // Create a new query object.
                $db             = $this->getDbo();
                $query  = $db->getQuery(true);

                // Select required fields from the categories.
                $query->select($this->getState('list.select', 'a.*'));
                $query->from('#__abbook AS a');
                $query->where('a.access IN ('.$groups.')');

                // Filter by category.
			$query->select('c.title AS cattitle, c.alias AS catalias');
                        $query->join('LEFT', '#__abcategories AS c ON c.id = a.catid');
                        $query->where('c.access IN ('.$groups.')');
                $query->join('LEFT', '#__abbooktag AS aa ON aa.idbook = a.id');
		$query->where('aa.idtag ='.$this->getState('tag.id'));
                // Filter by state
                $query->where('a.state = 1');
                // Filter by start and end dates.
                $nullDate = $db->Quote($db->getNullDate());
                $nowDate = $db->Quote(Factory::getDate()->toSql());

                // Filter by language
                if ($this->getState('filter.language')) {
                        $query->where('a.language in (' . $db->Quote(Factory::getLanguage()->getTag()) . ',' . $db->Quote('*') . ')');
		}

		// Add the list ordering clause.
		$query->order($db->escape($this->getState('list.ordering', 'a.ordering')).' '.$db->escape($this->getState('list.direction', 'ASC')));

		$query->group('a.id');

		return $query;
	}

	protected function _Authorname($idbook, $itemparams)
        {
		$db             = $this->getDbo();
                $query  = $db->getQuery(true);
                // Lets load the content if it doesn't already exist
		if ($this->state->params->get('name_display', 0) == 1){
                        $query->select('CONCAT(a.name, " ", a.lastname) AS author, a.id AS idauthor, a.alias');
                }else{
                        $query->select('CONCAT(a.lastname, " ", a.name) AS author, a.id AS idauthor, a.alias');
                }
		$query->select('CASE WHEN '.$query->charLength('a.alias', '!=', '0').' THEN '.$query->concatenate(array('a.id', 'a.alias'), ':'). ' ELSE a.id END as slugauthor');
                $query->from('#__abbookauth AS b');
                $query->innerjoin('#__abauthor AS a ON a.id = b.idauth');
                $query->where('b.idbook = '. (int) $idbook);
		//$query->order('a.lastname');
		$query->order($itemparams->get('author_order', 'a.lastname ASC'));
                $this->_db->setQuery($query);
                $this->_authorname = $this->_db->loadObjectList();
                return $this->_authorname;
        }
	/**
	 * Method to auto-populate the model state.
	 *
	 * Note. Calling getState in this method will result in recursion.
	 *
	 * @since	1.6
	 */
	protected function populateState($ordering = null, $direction = null)	
	{
		// Initialise variables.
		$app    = Factory::getApplication('site');

                // Load the parameters. Merge Global and Menu Item params into new object
                $params = $app->getParams();
                $menuParams = new JRegistry;

                if ($menu = $app->getMenu()->getActive()) {
                        $menuParams->loadString($menu->getParams());
                }
                $mergedParams = clone $menuParams;
                $mergedParams->merge($params);
		// Load the parameters.
                $this->setState('params', $mergedParams);

		// List state information
		$limit = $app->getUserStateFromRequest('abook.tag.list.limit', 'limit', $params->get('bookpagination'));
		$this->setState('list.limit', $limit);

		$limitstart = $app->input->getVar('limitstart', 0, '', 'int');
		$this->setState('list.start', $limitstart);

		$orderCol	= $app->input->getCmd('filter_order', $params->get('display_order','ordering'));
		$this->setState('list.ordering', $orderCol);

		$listOrder	=  $app->input->getCmd('filter_order_Dir', $params->get('books_display_order_dir','ASC'));
		$this->setState('list.direction', $listOrder);

		$id = $app->input->getInt('id');
		$this->setState('tag.id', $id);

		$this->setState('filter.published',	1);

		$this->setState('filter.language',$app->getLanguageFilter());
	}

	/**
	 * Method to get tag data for the current tag
	 *
	 * @param	int		An optional ID
	 *
	 * @return	object
	 * @since	1.5
	 */
	public function getTag()
	{
                if (!is_object($this->_item)) {
                        if( isset( $this->state->params ) ) {
                                $params = $this->state->params;
                                $options = array();
                                $options['countItems'] = $params->get('show_item_count', 0);
                        }
                        else {
                                $options['countItems'] = 0;
                        }
		$db             = $this->getDbo();
                $query  = $db->getQuery(true);
                $query->select('*');
                $query->from('#__abtag');
                $query->where('id = '. (int) $this->getState('tag.id'));
                $db->setQuery($query);
                $this->_item = $db->loadObject();


                }
                return $this->_item;
	}

	protected function _Tagname($idbook)
        {
		$db             = $this->getDbo();
                $query  = $db->getQuery(true);
		$query->select('a.name AS title, a.id AS tag_id, 1 AS access, "" AS params, a.alias');
		$query->from('#__abbooktag AS b');
		$query->innerjoin('#__abtag AS a ON a.id = b.idtag');
		$query->where('b.idbook = '. (int) $idbook);
		$query->order('a.name');
                $db->setQuery($query);
                $this->_tagname = $db->loadObjectList();
                return $this->_tagname;
        }

	function _Voting($idbook)
        {
		$db             = $this->getDbo();
                $query  = $db->getQuery(true);
                $query->select('ROUND( v.rating_sum / v.rating_count ) AS rating, v.rating_count');
                $query->from('#__abbook AS a');
                $query->leftjoin('#__abrating AS v ON a.id = v.book_id');
                $query->where('a.id='. (int) $idbook);
                $db->setQuery($query);
                $this->_voting = $db->loadAssoc();
                return $this->_voting;
        }
}
